/*
 * Copyright (C), 2000-2003 by Contributors to the monit codebase. 
 * All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include <config.h>

#include <stdio.h>
#include <stdlib.h>

#include "monitor.h"
#include "protocol.h"
#include "ssl.h"


/* Private prototypes */
static void _gcppl(Port_T*);
static void _gcpl(Process_T*);
static void _gcpcl(Checksum_T*);
static void _gcpql(Resource_T*);
static void _gcppil(ProcInfo_T*);
static void _gcptl(Timestamp_T*);
static void _gcpdl(Dependant_T *d);


/**
 *  Release allocated memory.
 *
 *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
 *  @author Martin Pala, <martin.pala@iol.cz>
 *
 *  @version \$Id: gc.c,v 1.16 2003/02/17 15:34:19 hauk Exp $
 *
 *  @file
 */


/* ------------------------------------------------------------------ Public */


void gc() {

  gc_protocols();
  if(processlist) _gcpl(&processlist);
  if(Run.mygroup) free(Run.mygroup);
  
}


void gc_process(Process_T *p) {

  int i;

  ASSERT(p);
  
  if((*p)->portlist) {
    
    _gcppl(&(*p)->portlist);

  }

  if((*p)->checksumlist) {
    
    _gcpcl(&(*p)->checksumlist);

  }

  if((*p)->maillist) {

    gc_mail_list(&(*p)->maillist);

  }

  if((*p)->resourcelist) {
    
    _gcpql(&(*p)->resourcelist);
    
  }

  if((*p)->procinfo) {

    _gcppil(&(*p)->procinfo);

  }
  
  if((*p)->timestamplist) {
    
    _gcptl(&(*p)->timestamplist);

  }

  if((*p)->dependantlist) {
    
    _gcpdl(&(*p)->dependantlist);
    
  }

  free((*p)->name);
  free((*p)->pidfile);
  free((*p)->group);
  if((*p)->start) {
    for(i= 0; (*p)->start->arg[i]; i++)
	free((*p)->start->arg[i]);
    free((*p)->start);
  }
  if((*p)->stop) {
    for(i= 0; (*p)->stop->arg[i]; i++)
	free((*p)->stop->arg[i]);
    free((*p)->stop);
  }
  (*p)->next= NULL;

  pthread_mutex_destroy(&(*p)->mutex);

  free(*p);

}
  

void gc_mail_list(Mail_T *m) {

  ASSERT(m);
  
  if((*m)->next)
      gc_mail_list(&(*m)->next);
  
  free((*m)->to);
  free((*m)->from);
  free((*m)->subject);
  free((*m)->message);
  free((*m)->opt_message);
  free(*m);
  *m= NULL;
  
}

    
/* ----------------------------------------------------------------- Private */


static void _gcpl(Process_T *p) {
  
  ASSERT(p);

  if((*p)->next) {
    
    _gcpl(&(*p)->next);
    
  }
  
  gc_process(&(*p));
  *p= NULL;
    
}


static void _gcppl(Port_T *p) {
  
  ASSERT(p);

  if((*p)->next) {
    
    _gcppl(&(*p)->next);
    
  }

  free((*p)->hostname);
  free((*p)->request);
  free((*p)->pathname);
  free((*p)->address);
  free((*p)->certmd5);
 
  delete_ssl_socket((*p)->ssl);
  free(*p);
  *p= NULL;

}


static void _gcpcl(Checksum_T *p) {
  
  ASSERT(p);

  if((*p)->next) {
    
    _gcpcl(&(*p)->next);
    
  }

  free((*p)->file); 
  free((*p)->md5);
  free(*p);
  *p= NULL;

}


static void _gcpql(Resource_T *q) {

  ASSERT(q);

  if((*q)->next) {

    _gcpql(&(*q)->next);

  }

  free(*q);
  *q= NULL;
  
}


static void _gcppil(ProcInfo_T *pi) {

  ASSERT(pi);

  free(*pi);
  *pi= NULL;

}


static void _gcptl(Timestamp_T *p) {
  
  ASSERT(p);

  if((*p)->next) {
    
    _gcptl(&(*p)->next);
    
  }

  free((*p)->pathname);
  free(*p);
  *p= NULL;

}


static void _gcpdl(Dependant_T *d) {

  ASSERT(d);

  if((*d)->next) {
    
    _gcpdl(&(*d)->next);
    
  }

  free((*d)->dependant);
  free(*d);
  *d= NULL;

}
