/*
 * Copyright (C), 2000-2003 by Contributors to the monit codebase. 
 * All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

/**
 *  System independent /proc & kvm methods.
 *
 *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
 *  @author Christian Hopp <chopp@iei.tu-clausthal.de>
 *
 *  @version \$Id: common.c,v 1.5 2003/02/11 21:27:40 martinp Exp $
 *
 *  @file
 */

#include <config.h>

#ifdef HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif

#ifdef HAVE_SYS_STAT_H
#include <sys/stat.h>
#endif

#ifdef HAVE_STRING_H
#include <string.h>
#endif

#ifdef HAVE_SYS_TIME_H
#include <sys/time.h>
#endif

#ifdef HAVE_LOADAVG_H
#include <sys/loadavg.h>
#endif

#ifdef HAVE_STRING_H
#include <string.h>
#endif

#ifdef HAVE_FCNTL_H
#include <fcntl.h>
#endif

#include <stdio.h>
#include <errno.h>


#include "process.h"


/**
 * Reads an process dependent entry or the proc files system
 * @param buf buffer to write to
 * @param buf_size size of buffer "buf"
 * @param name name of proc service
 * @param pid pid number of the process
 * @return TRUE if succeeded otherwise FALSE.
 */
int read_proc_file(char *buf, int buf_size, char * name, int pid) {

  int fd;
  char filename[STRLEN];
  int bytes;

  ASSERT(buf);
  ASSERT(name);

  snprintf(filename, STRLEN, "/proc/%d/%s", pid, name);

  fd = open(filename, O_RDONLY);

  if ( fd < 0 ) {

    return FALSE;

  }

  if ( (bytes = read(fd, buf, buf_size-1)) < 0 ) {

    return FALSE;

  }
       
  /* In case it is a string we have to 0 terminate it our self */
  buf[bytes]='\0';

  close(fd);

  return TRUE;
  
}

/**
 * Get the actual time as a floating point number
 * @return time in seconds
 */
double get_float_time(void) {    

  struct timeval t;

  gettimeofday(&t, NULL);
  return (double) t.tv_sec * 10 + (double) t.tv_usec / 100000.0;

}

/**
 * Get the proc infomation (CPU percentage, MEM in MByte and percent,
 * status)
 * @param p proc information structure to work with.
 * @return TRUE if succeeded otherwise FALSE.
 */
int get_process_info(ProcInfo_T p) {
  double temp_time;

  /* get the actual time */

  temp_time = get_float_time();
  
  /* get the system dependant proc data */

  if ( ! get_process_info_sysdep(p) ) {

    return FALSE;

  }

  /* check if it is the first run */
  
  if ( p->time_prev == 0.0 ) {
    
    p->cpu_percent= 0;
    
  } else {
    
    /* Calculate CPU percentage */

    p->cpu_percent=  (int) (( p->cputime - p->cputime_prev ) * 1000) / 
      ( temp_time - p->time_prev ) / num_cpus;
	
  }

  p->time_prev = temp_time;
    

  return TRUE;

}

