/*
 * Copyright (C), 2000-2003 by Contributors to the monit codebase. 
 * All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */


#include <config.h>

#ifdef HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif

#include <unistd.h>

#ifdef HAVE_SYS_STAT_H
#include <sys/stat.h>
#endif

#ifdef HAVE_FCNTL_H
#include <fcntl.h>
#endif

#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif

#ifdef TIME_WITH_SYS_TIME
#include <time.h>

#ifdef HAVE_SYS_TIME_H
#include <sys/time.h>
#endif
#else
#include <time.h>
#endif

#ifdef HAVE_STRING_H
#include <string.h>
#endif

#include <stdio.h>
#include <asm/param.h>
#include <asm/page.h>

#include "process.h"
#include "sysdep.h"

/**
 *  System dependent resource gathering code for Linux.
 *
 *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
 *  @author Christian Hopp <chopp@iei.tu-clausthal.de>
 *
 *  @version \$Id: sysdep_LINUX.c,v 1.2 2003/02/11 21:27:40 martinp Exp $
 *
 *  @file
 */

#define PAGE_TO_KBYTE_SHIFT PAGE_SHIFT-10

static long mem_kbyte_max;

int init_process_info_sysdep(void) {

  struct stat buf;
  

  /* I hope this is okay hack to get the total memsize. (-: */

  if ( stat("/proc/kcore", &buf) != 0 ) {

    return FALSE;

  }

  num_cpus= sysconf(_SC_NPROCESSORS_CONF);

  mem_kbyte_max = buf.st_size>>10;

  return TRUE;

}


int get_process_info_sysdep(ProcInfo_T p) {

  char buf[4096];
  char* tmp;
  char stat_item_state;
  unsigned long stat_item_utime;
  unsigned long stat_item_stime;
  long stat_item_cutime;
  long stat_item_cstime;
  long stat_item_rss;

  if (!read_proc_file(buf,4096, "stat", p->pid)) {

    return FALSE;

  }

  /* Move along the buffer to get past the process name */

  tmp = strrchr(buf, ')') + 2;

  /* This implementation is done by using fs/procfs/array.c as a basis
     it is also worth looking into the source of the procps utils */

  sscanf(tmp,"%c %*d %*d %*d %*d %*d %*u %*u"
	 "%*u %*u %*u %lu %lu %ld %ld %*d %*d %*d "
	 "%*d %*u %*u %ld %*u %*u %*u %*u %*u "
	 "%*u %*u %*u %*u %*u %*u %*u %*u %*d %*d\n",
	 &stat_item_state, &stat_item_utime, &stat_item_stime,
	 &stat_item_cutime, &stat_item_cstime, &stat_item_rss);

  /* abs to please the compiler... we dont want to shift negatively.
     why doesn't C understand this??? */

  if ( PAGE_TO_KBYTE_SHIFT < 0 ) {

    p->mem_kbyte= stat_item_rss >> abs(PAGE_TO_KBYTE_SHIFT);

  } else {

    p->mem_kbyte= stat_item_rss << abs(PAGE_TO_KBYTE_SHIFT);

  }

  p->mem_percent = (int) ((double) p->mem_kbyte * 1000.0 / mem_kbyte_max);

  /* jiffies -> seconds = 1 / HZ
     HZ is defined in "asm/param.h"  and it is usually 1/100s but on
     alpha system it is 1/1024s */

  p->cputime_prev = p->cputime;
  p->cputime =  ( stat_item_utime + stat_item_stime ) * 10 / HZ;

  if ( include_children ) {

    p->cputime += ( stat_item_cutime + stat_item_cstime ) * 10 / HZ;

  }

  /* first run ? */

  if ( p->time_prev == 0.0 ) {

      p->cputime_prev = p->cputime;

  }

  /* State is Zombie -> then we are a Zombie ... clear or? (-: */

  if ( stat_item_state == 'Z' ) {

    p->status_flag |= PROCESS_ZOMBIE;

  }

  return TRUE;

}
