/*
 * Copyright (C), 2000-2003 by Contributors to the monit codebase. 
 * All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */


#include <config.h>

#ifdef HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif

#include <unistd.h>

#ifdef HAVE_SYS_STAT_H
#include <sys/stat.h>
#endif

#ifdef HAVE_FCNTL_H
#include <fcntl.h>
#endif

#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif

#ifdef TIME_WITH_SYS_TIME
#include <time.h>

#ifdef HAVE_SYS_TIME_H
#include <sys/time.h>
#endif
#else
#include <time.h>
#endif

#ifdef HAVE_STRING_H
#include <string.h>
#endif

#include <stdio.h>
#include <procfs.h>

#include "process.h"
#include "sysdep.h"

/**
 *  System dependent resource gathering code for SOLARIS.
 *
 *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
 *  @author Christian Hopp <chopp@iei.tu-clausthal.de>
 *
 *  @version \$Id: sysdep_SOLARIS.c,v 1.2 2003/02/11 21:27:40 martinp Exp $
 *
 *  @file
 */


int init_process_info_sysdep(void) {

  num_cpus= sysconf( _SC_NPROCESSORS_ONLN);

  return (getuid()==0);

}

double timestruc_to_tseconds(timestruc_t t) {
  return  t.tv_sec * 10 + t.tv_nsec / 100000000.0;
}

int get_process_info_sysdep(ProcInfo_T p) {

  char buf[4096];
  psinfo_t  * psinfo=  (psinfo_t *)&buf;
  pstatus_t * pstatus= (pstatus_t *)&buf;

  if (!read_proc_file(buf,4096, "psinfo", p->pid)) {

    return FALSE;

  }

  /* If we don't have any light-weight processes (LWP) then we
     are definitely a zombie */

  if ( psinfo->pr_nlwp == 0 ) {

    p->status_flag = PROCESS_ZOMBIE;

  }


  if ( p->status_flag != PROCESS_ZOMBIE ) {
    /* We can't access /proc/$pid/status of a zombie */
    /* and does it anyway matter? */

    p->mem_percent = psinfo->pr_pctmem * 1000 / 0x8000;
    p->mem_kbyte = psinfo->pr_rssize;

    if (!read_proc_file(buf,4096, "status", p->pid)) {

      return FALSE;

    }

    p->cputime_prev= p->cputime;
    p->cputime= ( timestruc_to_tseconds(pstatus->pr_utime) +
		  timestruc_to_tseconds(pstatus->pr_stime) );

    if( include_children ) {

      p->cputime+= ( timestruc_to_tseconds(pstatus->pr_cutime) +
		     timestruc_to_tseconds(pstatus->pr_cstime) );

    }
    /* first run ? */

    if ( p->time_prev == 0.0 ) {

      p->cputime_prev= p->cputime;

    }

  } else {

    p->cputime_prev= p->cputime = 0;
    p->mem_kbyte= 0;
    p->mem_percent= 0.0;

  }

  return TRUE;

}
