/*
 * Copyright (C), 2000-2003 by Contributors to the monit codebase. 
 * All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include <config.h>

#include <stdio.h>
#include <errno.h>

#ifdef HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif

#include <sys/socket.h>

#ifdef HAVE_STRING_H
#include <string.h>
#endif

#ifdef HAVE_NETINET_IN_H
#include <netinet/in.h>
#endif

#ifdef HAVE_ARPA_INET_H
#include <arpa/inet.h>
#endif


#include "protocol.h"

/* Private prototypes */
static char *get_host_header(Port_T, char *);

/**
 *  A simple HTTP test.
 *
 *  We send the following request to the server:
 *  'GET / HTTP/1.1'             ... if request statement isn't defined
 *  'GET /custom/page  HTTP/1.1' ... if request statement is defined
 *  and check the server's status code.
 *
 *  If the host statement defines hostname, it is used in 'Host:'
 *  header.
 *
 *  If the status code is >= 400, an error has occurred.
 *  Return TRUE if the status code is OK, otherwise FALSE.
 *
 *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
 *  @author Martin Pala, <martin.pala@iol.cz>
 *
 *  @version \$Id: http.c,v 1.17 2003/02/14 08:22:33 martinp Exp $
 *
 *  @file
 */
int check_http(Port_T p) {

#define REQ_LENGTH  1024

  int n;
  int status;
  char buf[STRLEN];
  char msg[STRLEN];
  char host[STRLEN];
  char proto[STRLEN];
  char request[REQ_LENGTH];

  ASSERT(p);

  snprintf(request, REQ_LENGTH,
           "GET %s HTTP/1.1\r\n"
           "Host: %s\r\n"
	   "Accept: */*\r\n"
	   "User-Agent: %s/%s\r\n"
           "Connection: close\r\n\r\n",
           p->request?p->request:"/",
           get_host_header(p, host), prog, VERSION);

  if(port_send(p, request, strlen(request), 0) < 0) {
    log("HTTP: error sending data -- %s\n", STRERROR);
    return FALSE;
  }
  
  if(port_recv(p, buf, sizeof(buf), 0) <= 0) {
    log("HTTP: error receiving data -- %s\n", STRERROR);
    return FALSE;
  }

  chomp(buf);

  n= sscanf(buf, "%s %d %s", proto, &status, msg);
  if(n!=3 || (status >= 400)) {
    log("HTTP error: %s\n", buf);
    return FALSE;
  }

  return TRUE;
  
}


/* ----------------------------------------------------------------- Private */


/**
 * @return a "hostname:port" or a void string if host
 * equals LOCALHOST or if it is an IP address
 */
static char *get_host_header(Port_T p, char *hostbuf) {

  if(! strcmp(LOCALHOST, p->hostname) || inet_aton(p->hostname, NULL)) {

    *hostbuf= 0;

  } else {

    snprintf(hostbuf, STRLEN, "%s:%d", p->hostname, p->port);

  }

  return hostbuf;

}

