/*
 * Copyright (C), 2000-2003 by Contributors to the monit codebase. 
 * All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include <config.h>

#include <stdio.h>
#include <errno.h>

#ifdef HAVE_STRING_H
#include <string.h>
#endif

#include "protocol.h"

/**
 *  Check the server for greeting code +OK, then send QUIT and check
 *  for code +OK. If alive return TRUE, else, return FALSE.
 *
 *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
 *
 *  @version \$Id: pop.c,v 1.9 2003/02/11 21:27:40 martinp Exp $
 *
 *  @file
 */
int check_pop(Port_T p) {

  char buf[STRLEN];
  const char *ok= "+OK";
  const char *command= "QUIT\r\n";

  ASSERT(p);
  
  if(port_recv(p, buf, sizeof(buf), 0) <= 0) {
    log("POP: error receiving data -- %s\n", STRERROR);
    return FALSE;
  }

  chomp(buf);
  
  if(strncasecmp(buf, ok, strlen(ok)) != 0) {
    log("POP error: %s\n", buf);
    return FALSE;
  }
  
  if(port_send(p, command, strlen(command), 0) < 0) {
    log("POP: error sending data -- %s\n", STRERROR);
    return FALSE;
  }

  if(port_recv(p, buf, sizeof(buf), 0) <= 0) {
    log("POP: error receiving data -- %s\n", STRERROR);
    return FALSE;
  }

  chomp(buf);
  
  if(strncasecmp(buf, ok, strlen(ok)) != 0) {
    log("POP error: %s\n", buf);
    return FALSE;
  }

  return TRUE;
  
}

