/*
 * Copyright (C), 2000-2003 by Contributors to the monit codebase. 
 * All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include <config.h>

#include <stdio.h>
#include <stdlib.h>

#ifdef HAVE_STRING_H
#include <string.h>
#endif

#include "protocol.h"

/* Private variables */
static Protocol_T mydefault= NULL;
static Protocol_T myhttp= NULL;
static Protocol_T myftp= NULL;
static Protocol_T mysmtp= NULL;
static Protocol_T mypop= NULL;
static Protocol_T myimap= NULL;
static Protocol_T mynntp= NULL;
static Protocol_T myssh= NULL;
static Protocol_T mydwp= NULL;
static Protocol_T myldap2= NULL;
static Protocol_T myldap3= NULL;


/**
 *  Factory module for vending protocol objects. Using lazy
 *  initialization, and dishing out only one copy of the object.
 *
 *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
 *
 *  @version \$Id: protocol.c,v 1.9 2003/02/11 21:27:40 martinp Exp $
 *
 *  @file
 */


/* ------------------------------------------------------------------ Public */


void gc_protocols() {

  free(mydefault); mydefault= NULL;
  free(myhttp);    myhttp= NULL;
  free(myftp);     myftp= NULL;
  free(mysmtp);    mysmtp= NULL;
  free(mypop);     mypop= NULL;
  free(myimap);    myimap= NULL;
  free(mynntp);    mynntp= NULL;
  free(myssh);     myssh= NULL;
  free(mydwp);     mydwp= NULL;
  free(myldap2);   myldap2= NULL;
  free(myldap3);   myldap3= NULL;

}


void *create_default() {

  if(mydefault == NULL) {
    static const char *name= "DEFAULT";
    mydefault= NEW(mydefault);
    mydefault->name= name;
    mydefault->check= check_default;
  }

  return mydefault;

}


void *create_http() {

  if(myhttp == NULL) {
    static const char *name= "HTTP";
    myhttp= NEW(myhttp);
    myhttp->name= name;
    myhttp->check= check_http;
  }

  return myhttp;

}


void *create_ftp() {

  if(myftp == NULL) {
    static const char *name= "FTP";
    myftp= NEW(myftp);
    myftp->name= name;
    myftp->check= check_ftp;
  }

  return myftp;

}


void *create_smtp() {

  if(mysmtp == NULL) {
    static const char *name= "SMTP";
    mysmtp= NEW(mysmtp);
    mysmtp->name= name;
    mysmtp->check= check_smtp;
  }

  return mysmtp;

}


void *create_pop() {

  if(mypop == NULL) {
    static const char *name= "POP";
    mypop= NEW(mypop);
    mypop->name= name;
    mypop->check= check_pop;
  }

  return mypop;

}


void *create_imap() {

  if(myimap == NULL) {
    static const char *name= "IMAP";
    myimap= NEW(myimap);
    myimap->name= name;
    myimap->check= check_imap;
  }

  return myimap;

}


void *create_nntp() {

  if(mynntp == NULL) {
    static const char *name= "NNTP";
    mynntp= NEW(mynntp);
    mynntp->name= name;
    mynntp->check= check_nntp;
  }

  return mynntp;

}


void *create_ssh() {
  
  if(myssh == NULL) {
    static const char *name= "SSH";
    myssh= NEW(myssh);
    myssh->name= name;
    myssh->check= check_ssh;
  }
  
  return myssh;

}


void *create_dwp() {
  
  if(mydwp == NULL) {
    static const char *name= "DWP";
    mydwp= NEW(mydwp);
    mydwp->name= name;
    mydwp->check= check_dwp;
  }
  
  return mydwp;

}


void *create_ldap2() {
  
  if(myldap2 == NULL) {
    static const char *name= "LDAP2";
    myldap2= NEW(myldap2);
    myldap2->name= name;
    myldap2->check= check_ldap2;
  }
  
  return myldap2;

}


void *create_ldap3() {
  
  if(myldap3 == NULL) {
    static const char *name= "LDAP3";
    myldap3= NEW(myldap3);
    myldap3->name= name;
    myldap3->check= check_ldap3;
  }
  
  return myldap3;

}

